package com.io.yutian.aulib.sql.builder.impl;

import com.io.yutian.aulib.sql.api.SQLAction;
import com.io.yutian.aulib.sql.api.builder.InsertBuilder;
import com.io.yutian.aulib.sql.builder.AbstractSQLBuilder;
import com.io.yutian.aulib.sql.manager.SQLManagerImpl;
import org.jetbrains.annotations.NotNull;

import java.util.Iterator;
import java.util.List;
import java.util.Objects;

import static com.io.yutian.aulib.sql.api.SQLBuilder.withBackQuote;

public abstract class InsertBuilderImpl<T extends SQLAction<?>>
        extends AbstractSQLBuilder implements InsertBuilder<T> {

    protected final String tableName;
    protected boolean ignore;

    public InsertBuilderImpl(@NotNull SQLManagerImpl manager, String tableName) {
        super(manager);
        Objects.requireNonNull(tableName);
        this.tableName = tableName;
    }

    @Override
    public boolean isIgnore() {
        return ignore;
    }

    @Override
    public InsertBuilder<T> setIgnore(boolean ignore) {
        this.ignore = ignore;
        return this;
    }

    protected static String buildSQL(String tableName, boolean ignore, List<String> columnNames) {
        return buildSQL("INSERT "+(ignore ? "IGNORE " : "")+"INTO", tableName, columnNames);
    }

    protected static String buildSQL(String sqlPrefix, String tableName, List<String> columnNames) {
        int valueLength = columnNames.size();
        StringBuilder sqlBuilder = new StringBuilder();

        sqlBuilder.append(sqlPrefix).append(" ").append(withBackQuote(tableName)).append("(");
        Iterator<String> iterator = columnNames.iterator();
        while (iterator.hasNext()) {
            sqlBuilder.append(withBackQuote(iterator.next()));
            if (iterator.hasNext()) sqlBuilder.append(", ");
        }

        sqlBuilder.append(") VALUES (");

        for (int i = 0; i < valueLength; i++) {
            sqlBuilder.append("?");
            if (i != valueLength - 1) {
                sqlBuilder.append(", ");
            }
        }
        sqlBuilder.append(")");
        return sqlBuilder.toString();
    }

    @Override
    public String getTableName() {
        return tableName;
    }
}
